#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
"""
Module for handling /tokens APIs
"""
import connexion

from fabric_cm.credmgr.credential_managers.oauth_credmgr import OAuthCredmgr
from fabric_cm.credmgr.swagger_server.models.request import Request  # noqa: E501
from fabric_cm.credmgr.swagger_server.models.success import Success  # noqa: E501
from fabric_cm.credmgr.swagger_server import received_counter, success_counter, failure_counter
from fabric_cm.credmgr.swagger_server.response.constants import HTTP_METHOD_POST, \
    TOKENS_REVOKE_URL, TOKENS_REFRESH_URL, \
    TOKENS_CREATE_URL, VOUCH_ID_TOKEN, VOUCH_REFRESH_TOKEN, AUTHORIZATION_ERR
from fabric_cm.credmgr.config import CONFIG_OBJ
from fabric_cm.credmgr.logging import LOG

def authorize(request):
    ci_logon_id_token = request.headers.get(VOUCH_ID_TOKEN, None)
    refresh_token = request.headers.get(VOUCH_REFRESH_TOKEN, None)
    cookie_name = CONFIG_OBJ.get_vouch_cookie_name()
    cookie = request.cookies.get(cookie_name)

    return ci_logon_id_token, refresh_token, cookie


def tokens_create_post(project_name=None, scope=None):  # noqa: E501
    """Generate Fabric OAuth tokens for an user

    Request to generate Fabric OAuth tokens for an user  # noqa: E501

    :param project_name: Project Name
    :type project_name: str
    :param scope: Scope for which token is requested
    :type scope: str

    :rtype: Success
    """
    received_counter.labels(HTTP_METHOD_POST, TOKENS_CREATE_URL).inc()
    try:
        ci_logon_id_token, refresh_token, cookie = authorize(connexion.request)
        if ci_logon_id_token is None:
            return AUTHORIZATION_ERR, 401

        credmgr = OAuthCredmgr()
        result = credmgr.create_token(ci_logon_id_token=ci_logon_id_token,
                                      refresh_token=refresh_token,
                                      project=project_name,
                                      scope=scope,
                                      cookie=cookie)
        response = Success.from_dict(result)
        LOG.debug(result)
        success_counter.labels(HTTP_METHOD_POST, TOKENS_CREATE_URL).inc()
        return response
    except Exception as ex:
        LOG.exception(ex)
        failure_counter.labels(HTTP_METHOD_POST, TOKENS_CREATE_URL).inc()
        return str(ex), 500


def tokens_refresh_post(body, project_name=None, scope=None):  # noqa: E501
    """Refresh FABRIC OAuth tokens for an user

    Request to refresh OAuth tokens for an user  # noqa: E501

    :param body:
    :type body: dict | bytes
    :param project_name: Project Name
    :type project_name: str
    :param scope: Scope for which token is requested
    :type scope: str

    :rtype: Success
    """
    received_counter.labels(HTTP_METHOD_POST, TOKENS_REFRESH_URL).inc()
    if connexion.request.is_json:
        body = Request.from_dict(connexion.request.get_json())  # noqa: E501
    try:
        ci_logon_id_token, refresh_token, cookie = authorize(connexion.request)
        credmgr = OAuthCredmgr()
        response = Success.from_dict(credmgr.refresh_token(refresh_token=body.refresh_token,
                                                           project=project_name, scope=scope,
                                                           cookie=cookie))
        success_counter.labels(HTTP_METHOD_POST, TOKENS_REFRESH_URL).inc()
        return response
    except Exception as ex:
        LOG.exception(ex)
        failure_counter.labels(HTTP_METHOD_POST, TOKENS_REFRESH_URL).inc()
        return str(ex), 500


def tokens_revoke_post(body):  # noqa: E501
    """Revoke a refresh token for an user

    Request to revoke a refresh token for an user  # noqa: E501

    :param body:
    :type body: dict | bytes

    :rtype: Success
    """
    received_counter.labels(HTTP_METHOD_POST, TOKENS_REVOKE_URL).inc()
    if connexion.request.is_json:
        body = Request.from_dict(connexion.request.get_json())  # noqa: E501
    try:
        credmgr = OAuthCredmgr()
        credmgr.revoke_token(refresh_token=body.refresh_token)
        success_counter.labels(HTTP_METHOD_POST, TOKENS_REVOKE_URL).inc()
    except Exception as ex:
        LOG.exception(ex)
        failure_counter.labels(HTTP_METHOD_POST, TOKENS_REVOKE_URL).inc()
        return str(ex), 500
    return {}



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
from datetime import datetime
from dateutil import tz
from fss_utils.jwt_manager import JWTManager, ValidateCode
from fss_utils.vouch_encoder import VouchEncoder, CustomClaimsType, PTokens

from fabric_cm.credmgr.config import CONFIG_OBJ
from fabric_cm.credmgr.logging import LOG
from fabric_cm.credmgr.common.exceptions import TokenError
from fabric_cm.credmgr.external_apis.project_registry import ProjectRegistry


class FabricTokenEncoder:
    """
    Implements class to transform CILogon ID token to Fabric Id Token
    by adding the project, scope and membership information to the token
    and signing with Fabric Certificate
    """
    def __init__(self, id_token, idp_claims: dict, project="all", scope="all", cookie: str = None):
        """
        Constructor
        :param id_token: CI Logon Identity Token
        :param idp_claims: CI Logon Identity Claims
        :param project: Project for which token is requested
        :param scope: Scope for which token is requested
        :param cookie: Vouch Proxy Cookie

        :raises Exception in case of error
        """
        if id_token is None or project is None or scope is None:
            raise TokenError("Missing required parameters id_token or project or scope")

        LOG.debug("id_token %s", id_token)
        self.id_token = id_token
        self.claims = idp_claims
        self.project = project
        self.scope = scope
        self.cookie = cookie
        self.encoded = False
        self.token = None
        self.unset = True

    def encode(self, private_key: str, validity_in_seconds: int, kid: str, pass_phrase: str = None) -> str:
        """
        Generate Fabric Token by adding additional claims and signing with Fabric Cert
        :param private_key Private Key to sign the fabric_cm token
        :param validity_in_seconds Validity of the Token in seconds
        :param kid Public Key Id
        :param pass_phrase Pass Phrase for Private Key
        :return JWT String containing encoded Fabric Token
        """
        if self.encoded:
            return self.token

        self._add_fabric_claims()

        code, token_or_exception = JWTManager.encode_and_sign_with_private_key(validity=validity_in_seconds,
                                                                               claims=self.claims,
                                                                               private_key_file_name=private_key,
                                                                               pass_phrase=pass_phrase, kid=kid,
                                                                               algorithm='RS256')
        if code != ValidateCode.VALID:
            LOG.error(f"Failed to encode the Fabric Token: {token_or_exception}")
            raise token_or_exception

        self.token = token_or_exception
        self.encoded = True
        return self.token

    def _get_vouch_cookie(self) -> str:
        vouch_cookie_enabled = CONFIG_OBJ.is_vouch_cookie_enabled()
        if not vouch_cookie_enabled or self.cookie is not None:
            return self.cookie

        vouch_secret = CONFIG_OBJ.get_vouch_secret()
        vouch_compression = CONFIG_OBJ.is_vouch_cookie_compressed()
        vouch_claims = CONFIG_OBJ.get_vouch_custom_claims()
        vouch_cookie_lifetime = CONFIG_OBJ.get_vouch_cookie_lifetime()
        vouch_helper = VouchEncoder(secret=vouch_secret, compression=vouch_compression)

        custom_claims = []
        for c in vouch_claims:
            c_type = c.strip().upper()

            if c_type == CustomClaimsType.OPENID.name:
                custom_claims.append(CustomClaimsType.OPENID)

            if c_type == CustomClaimsType.EMAIL.name:
                custom_claims.append(CustomClaimsType.EMAIL)

            if c_type == CustomClaimsType.PROFILE.name:
                custom_claims.append(CustomClaimsType.PROFILE)

            if c_type == CustomClaimsType.CILOGON_USER_INFO.name:
                custom_claims.append(CustomClaimsType.CILOGON_USER_INFO)

        p_tokens = PTokens(id_token=self.id_token, idp_claims=self.claims)

        code, cookie_or_exception = vouch_helper.encode(custom_claims_type=custom_claims, p_tokens=p_tokens,
                                                        validity_in_seconds=vouch_cookie_lifetime)

        if code != ValidateCode.VALID:
            LOG.error(f"Failed to encode the Vouch Cookie: {cookie_or_exception}")
            raise cookie_or_exception

        return cookie_or_exception

    def _add_fabric_claims(self):
        """
        Set the claims for the Token by adding membership, project and scope
        """
        sub = self.claims.get("sub")
        url = CONFIG_OBJ.get_pr_url()
        project_registry = ProjectRegistry(api_server=url, cookie=self._get_vouch_cookie(),
                                           cookie_name=CONFIG_OBJ.get_vouch_cookie_name(),
                                           cookie_domain=CONFIG_OBJ.get_vouch_cookie_domain_name())
        roles, projects = project_registry.get_projects_and_roles(sub)

        LOG.debug("Projects: %s, Roles: %s", projects, roles)

        projects_to_be_removed = []
        for project in projects.keys():
            LOG.debug("Processing %s", project)
            if self.project != "all" and self.project not in project:
                projects_to_be_removed.append(project)
        for x in projects_to_be_removed:
            projects.pop(x)

        if len(projects) < 1:
            raise TokenError("User is not a member of any of the project")
        self.claims['projects'] = projects
        self.claims["roles"] = roles
        self.claims["scope"] = self.scope
        LOG.debug("Claims %s", self.claims)
        self.unset = False

    @staticmethod
    def get_local_from_utc(utc: int) -> datetime:
        """ convert UTC in claims (iat and exp) into a python
        datetime object """
        return datetime.fromtimestamp(utc, tz.tzlocal())

    def __repr__(self) -> str:
        return self.__str__()

    def __str__(self) -> str:
        if self.unset:
            return "JWT not initialized"

        fstring = f"Token for {self.claims['sub']}/{self.claims['name']}:"

        if 'iat' in self.claims:
            fstring += f"\n\tIssued on: " \
                       f"{self.get_local_from_utc(self.claims['iat']).strftime('%Y-%m-%d %H:%M:%S')}"
        if 'exp' in self.claims:
            fstring += f"\n\tExpires on: " \
                       f"{self.get_local_from_utc(self.claims['exp']).strftime('%Y-%m-%d %H:%M:%S')}"

        return fstring



# coding: utf-8

from __future__ import absolute_import

from flask import json
from six import BytesIO

from fabric_cm.credmgr.swagger_server.models.request import Request  # noqa: E501
from fabric_cm.credmgr.swagger_server.models.success import Success  # noqa: E501
from fabric_cm.credmgr.swagger_server.test import BaseTestCase


class TestTokensController(BaseTestCase):
    """TokensController integration test stubs"""

    def test_tokens_create_post(self):
        """Test case for tokens_create_post

        Generate tokens for an user
        """
        query_string = [('project_name', 'all'),
                        ('scope', 'all')]
        response = self.client.open(
            '//tokens/create',
            method='POST',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_tokens_refresh_post(self):
        """Test case for tokens_refresh_post

        Refresh tokens for an user
        """
        body = Request()
        query_string = [('project_name', 'all'),
                        ('scope', 'all')]
        response = self.client.open(
            '//tokens/refresh',
            method='POST',
            data=json.dumps(body),
            content_type='application/json',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_tokens_revoke_post(self):
        """Test case for tokens_revoke_post

        Revoke a refresh token for an user
        """
        body = Request()
        response = self.client.open(
            '//tokens/revoke',
            method='POST',
            data=json.dumps(body),
            content_type='application/json')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))


if __name__ == '__main__':
    import unittest
    unittest.main()



# coding: utf-8

# flake8: noqa
from __future__ import absolute_import
# import models into model package
from fabric_cm.credmgr.swagger_server.models.jwks import Jwks
from fabric_cm.credmgr.swagger_server.models.jwks_keys import JwksKeys
from fabric_cm.credmgr.swagger_server.models.request import Request
from fabric_cm.credmgr.swagger_server.models.success import Success
from fabric_cm.credmgr.swagger_server.models.version import Version



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
import configparser
from datetime import datetime
from typing import List

from fabric_cm.credmgr.common.exceptions import ConfigError


class Config:
    # Sections
    SECTION_RUNTIME = 'runtime'
    SECTION_LOGGING = 'logging'
    SECTION_OAUTH = 'oauth'
    SECTION_LDAP = 'ldap'
    SECTION_JWT = 'jwt'
    SECTION_PROJECT_REGISTRY = 'project-registry'
    SECTION_VOUCH = 'vouch'

    # Runtime parameters
    REST_PORT = 'rest-port'
    PROMETHEUS_PORT = 'prometheus-port'
    ENABLE_PROJECT_REGISTRY = 'enable-project-registry'
    ENABLE_VOUCH_COOKIE = 'enable-vouch-cookie'
    TOKEN_LIFETIME = 'token-lifetime'
    PROJECT_NAMES_IGNORE_LIST = 'prject-names-ignore-list'
    ROLES_LIST = 'roles-list'
    ALLOWED_SCOPES = 'allowed-scopes'

    # Logging Parameters
    LOGGER = 'logger'
    LOG_DIR = 'log-directory'
    LOG_FILE = 'log-file'
    LOG_RETAIN = 'log-retain'
    LOG_SIZE = 'log-size'
    LOG_LEVEL = 'log-level'

    # Oauth Parameters
    PROVIDER = 'oauth-provider'
    TOKEN_URL = 'oauth-token-url'
    REVOKE_URL = 'oauth-revoke-url'
    JWKS_URL = 'oauth-jwks-url'
    KEY_REFRESH = 'oauth-key-refresh'
    CLIENT_ID = 'oauth-client-id'
    CLIENT_SECRET = 'oauth-client-secret'

    # LDAP Parameters
    LDAP_HOST = 'ldap-host'
    LDAP_USER = 'ldap-user'
    LDAP_PASSWORD = 'ldap-password'
    LDAP_SEARCH_BASE = 'ldap-search-base'

    # JWT Parameters
    JWT_PUBLIC_KEY = 'jwt-public-key'
    JWT_PUBLIC_KEY_KID = 'jwt-public-key-kid'
    JWT_PRIVATE_KEY = 'jwt-private-key'
    JWT_PRIVATE_KEY_PASS_PHRASE = 'jwt-pass-phrase'

    # Project Registry Parameters
    PROJECT_REGISTRY_URL = 'project-registry-url'
    SSL_VERIFY = 'ssl_verify'

    # Vouch Parameters
    VOUCH = 'vouch'
    SECRET = 'secret'
    COMPRESSION = 'compression'
    CUSTOM_CLAIMS = 'custom_claims'
    LIFETIME = 'lifetime'
    COOKIE_NAME = 'cookie-name'
    COOKIE_DOMAIN_NAME = 'cookie-domain-name'

    def __init__(self, config_parser: configparser.ConfigParser):
        self.config_parser = config_parser

    def _get_config_from_section(self, section_name: str, parameter_name: str) -> str:
        try:
            return self.config_parser.get(section_name, parameter_name)
        except Exception as e:
            raise ConfigError("Missing {} in section {} Error: {}".format(parameter_name, section_name, e))

    def get_rest_port(self) -> int:
        return int(self._get_config_from_section(self.SECTION_RUNTIME, self.REST_PORT))

    def get_prometheus_port(self) -> int:
        return int(self._get_config_from_section(self.SECTION_RUNTIME, self.PROMETHEUS_PORT))

    def is_project_registry_enabled(self) -> bool:
        value = self._get_config_from_section(self.SECTION_RUNTIME, self.ENABLE_PROJECT_REGISTRY)
        if value.lower() == 'true':
            return True
        return False

    def is_vouch_cookie_enabled(self) -> bool:
        value = self._get_config_from_section(self.SECTION_RUNTIME, self.ENABLE_VOUCH_COOKIE)
        if value.lower() == 'true':
            return True
        return False

    def get_allowed_scopes(self) -> str:
        return self._get_config_from_section(self.SECTION_RUNTIME, self.ALLOWED_SCOPES)

    def get_roles(self) -> str:
        return self._get_config_from_section(self.SECTION_RUNTIME, self.ROLES_LIST)

    def get_project_ignore_list(self) -> str:
        return self._get_config_from_section(self.SECTION_RUNTIME, self.PROJECT_NAMES_IGNORE_LIST)

    def get_token_life_time(self) -> int:
        return int(self._get_config_from_section(self.SECTION_RUNTIME, self.TOKEN_LIFETIME))

    def get_logger_name(self) -> str:
        return self._get_config_from_section(self.SECTION_LOGGING, self.LOGGER)

    def get_logger_dir(self) -> str:
        return self._get_config_from_section(self.SECTION_LOGGING, self.LOG_DIR)

    def get_logger_file(self) -> str:
        return self._get_config_from_section(self.SECTION_LOGGING, self.LOG_FILE)

    def get_logger_level(self) -> str:
        return self._get_config_from_section(self.SECTION_LOGGING, self.LOG_LEVEL)

    def get_logger_retain(self) -> int:
        return int(self._get_config_from_section(self.SECTION_LOGGING, self.LOG_RETAIN))

    def get_logger_size(self) -> int:
        return int(self._get_config_from_section(self.SECTION_LOGGING, self.LOG_SIZE))

    def get_jwt_public_key(self) -> str:
        return self._get_config_from_section(self.SECTION_JWT, self.JWT_PUBLIC_KEY)

    def get_jwt_public_key_kid(self) -> str:
        return self._get_config_from_section(self.SECTION_JWT, self.JWT_PUBLIC_KEY_KID)

    def get_jwt_private_key(self) -> str:
        return self._get_config_from_section(self.SECTION_JWT, self.JWT_PRIVATE_KEY)

    def get_jwt_private_key_pass_phrase(self) -> str:
        return self._get_config_from_section(self.SECTION_JWT, self.JWT_PRIVATE_KEY_PASS_PHRASE)

    def get_oauth_provider(self) -> str:
        return self._get_config_from_section(self.SECTION_OAUTH, self.PROVIDER)

    def get_oauth_token_url(self) -> str:
        return self._get_config_from_section(self.SECTION_OAUTH, self.TOKEN_URL)

    def get_oauth_jwks_url(self) -> str:
        return self._get_config_from_section(self.SECTION_OAUTH, self.JWKS_URL)

    def get_oauth_revoke_url(self) -> str:
        return self._get_config_from_section(self.SECTION_OAUTH, self.REVOKE_URL)

    def get_oauth_client_id(self) -> str:
        return self._get_config_from_section(self.SECTION_OAUTH, self.CLIENT_ID)

    def get_oauth_client_secret(self) -> str:
        return self._get_config_from_section(self.SECTION_OAUTH, self.CLIENT_SECRET)

    def get_oauth_key_refresh(self) -> datetime:
        value = self._get_config_from_section(self.SECTION_OAUTH, self.KEY_REFRESH)
        return datetime.strptime(value, "%H:%M:%S")

    def get_ldap_host(self):
        return self._get_config_from_section(self.SECTION_LDAP, self.LDAP_HOST)

    def get_ldap_user(self):
        return self._get_config_from_section(self.SECTION_LDAP, self.LDAP_USER)

    def get_ldap_pwd(self):
        return self._get_config_from_section(self.SECTION_LDAP, self.LDAP_PASSWORD)

    def get_ldap_search_base(self):
        return self._get_config_from_section(self.SECTION_LDAP, self.LDAP_SEARCH_BASE)

    def is_pr_ssl_verify(self) -> bool:
        value = self._get_config_from_section(self.SECTION_PROJECT_REGISTRY, self.SSL_VERIFY)
        if value.lower() == 'true':
            return True
        return False

    def get_pr_url(self) -> str:
        return self._get_config_from_section(self.SECTION_PROJECT_REGISTRY, self.PROJECT_REGISTRY_URL)

    def get_vouch_secret(self) -> str:
        return self._get_config_from_section(self.SECTION_VOUCH, self.SECRET)

    def is_vouch_cookie_compressed(self) -> bool:
        value = self._get_config_from_section(self.SECTION_VOUCH, self.COMPRESSION)
        if value.lower() == 'true':
            return True
        return False

    def get_vouch_custom_claims(self) -> List[str]:
        value = self._get_config_from_section(self.SECTION_VOUCH, self.CUSTOM_CLAIMS)
        return value.split(',')

    def get_vouch_cookie_lifetime(self) -> int:
        return int(self._get_config_from_section(self.SECTION_VOUCH, self.LIFETIME))

    def get_vouch_cookie_name(self) -> str:
        return self._get_config_from_section(self.SECTION_VOUCH, self.COOKIE_NAME)

    def get_vouch_cookie_domain_name(self) -> str:
        return self._get_config_from_section(self.SECTION_VOUCH, self.COOKIE_DOMAIN_NAME)

    def get_providers(self) -> dict:
        """
        Constructor providers dict based on the information provided in config file
        """
        providers = {}
        provider = self.get_oauth_provider()
        providers[provider] = {}
        providers[provider]['client_id'] = self.get_oauth_client_id()
        providers[provider]['client_secret'] = self.get_oauth_client_secret()
        providers[provider]['token_uri'] = self.get_oauth_token_url()
        providers[provider]['revoke_uri'] = self.get_oauth_revoke_url()

        return providers



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
"""
Module for handling version APIs
"""
import requests

from fabric_cm.credmgr.swagger_server.models.jwks import Jwks
from fabric_cm.credmgr.swagger_server.models.version import Version  # noqa: E501
from fabric_cm.credmgr.swagger_server import received_counter, success_counter, failure_counter, fabric_jwks
from fabric_cm.credmgr.swagger_server.response.constants import VERSION_URL, HTTP_METHOD_GET, CERTS_URL
from fabric_cm.credmgr.logging import LOG

def version_get():  # noqa: E501
    """version

    Version # noqa: E501


    :rtype: Version
    """
    received_counter.labels(HTTP_METHOD_GET, VERSION_URL).inc()
    try:
        version = '1.0.0'
        tag = '1.0.0'
        url = "https://api.github.com/repos/fabric-testbesd/CredentialManager/git/refs/tags/{}".format(tag)

        response = Version()
        response.version = version
        response.gitsha1 = 'Not Available'

        result = requests.get(url)
        if result.status_code == 200 and result.json() is not None:
            object_json = result.json().get("object", None)
            if object_json is not None:
                sha = object_json.get("sha", None)
                if sha is not None:
                    response.gitsha1 = sha
        success_counter.labels(HTTP_METHOD_GET, VERSION_URL).inc()
    except Exception as ex:
        LOG.exception(ex)
        failure_counter.labels(HTTP_METHOD_GET, VERSION_URL).inc()
        return str(ex), 500
    return response


def certs_get():  # noqa: E501
    """Return Public Keys to verify signature of the tokens

    Json Web Keys # noqa: E501


    :rtype: List[Jwk]
    """
    received_counter.labels(HTTP_METHOD_GET, CERTS_URL).inc()
    try:
        response = Jwks.from_dict(fabric_jwks)
        LOG.debug(response)
        success_counter.labels(HTTP_METHOD_GET, CERTS_URL).inc()
        return response
    except Exception as ex:
        LOG.exception(ex)
        failure_counter.labels(HTTP_METHOD_GET, CERTS_URL).inc()
        return str(ex), 500



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
import requests

from fabric_cm.credmgr.config import CONFIG_OBJ
from fabric_cm.credmgr.logging import LOG


class ProjectRegistry:
    """
    Class implements functionality to interface with Project Registry
    """
    def __init__(self, api_server: str, cookie: str, cookie_name: str, cookie_domain: str):
        self.api_server = api_server
        self.cookie = cookie
        self.cookie_name = cookie_name
        self.cookie_domain = cookie_domain

    def get_projects_and_roles(self, sub: str):
        """
        Determine Role from Project Registry
        :param sub: OIDC claim sub
        :param returns the roles and project with tags

        :returns a tuple containing user specific roles and project tags
        """
        if self.api_server is None or self.cookie is None:
            raise ProjectRegistryError(f"Project Registry URL: {self.api_server} or "
                                       "Cookie: {self.cookie} not available")

        # Create Session
        s = requests.Session()

        # Set the Cookie
        cookie_obj = requests.cookies.create_cookie(
            name=self.cookie_name,
            value=self.cookie
        )
        s.cookies.set_cookie(cookie_obj)
        LOG.debug(f"Using vouch cookie: {s.cookies}")

        # Set the headers
        headers = {
            'Accept': 'application/json',
            'Content-Type': "application/json"
        }
        s.headers.update(headers)

        # Get User by OIDC SUB Claim
        url = self.api_server + "/people/oidc_claim_sub?oidc_claim_sub={}".format(sub)
        ssl_verify = CONFIG_OBJ.is_pr_ssl_verify()
        response = s.get(url, verify=ssl_verify)

        if response.status_code != 200:
            raise ProjectRegistryError(f"Project Registry error occurred "
                                       f"status_code: {response.status_code} message: {response.content}")

        LOG.debug(f"Response : {response.json()}")

        roles = response.json().get('roles', None)
        projects = response.json().get('projects', None)

        # Get Per Project Tags
        project_tags = {}
        for p in projects:
            project_name = p.get('name', None)
            project_uuid = p.get('uuid', None)
            LOG.debug(f"Getting tags for Project: {project_name}")
            url = self.api_server + "/projects/{}".format(project_uuid)
            response = s.get(url, verify=ssl_verify)
            if response.status_code != 200:
                raise ProjectRegistryError(f"Project Registry error occurred "
                                           f"status_code: {response.status_code} message: {response.content}")
            project_tags[project_name] = response.json().get('tags', None)
        return roles, project_tags


class ProjectRegistryError(Exception):
    """
    Project Registry Exception
    """
    pass



import unittest
import requests


class TestCredmgr(unittest.TestCase):
    """
    Test Credential Manager
    """
    base_url = "https://localhost:8443/tokens/"
    create_url = base_url + "create"
    refresh_url = base_url + "refresh"
    revoke_url = base_url + "revoke"

    missing_param_str = 'Missing required parameters id_token or project or scope'
    missing_authorization = 'Authorization information is missing or invalid: /tokens'

    headers = {
        'Accept': 'application/json',
    }

    def test_create_tokens(self):
        """
        Verify token creation is successful with default project and scope
        Verify get token without authentication fails
        :return:
        """
        # Create tokens with default project and scope
        response = requests.post(url=self.create_url, headers=self.headers, verify=False)
        self.assertEqual(401, response.status_code)
        self.assertEqual(response.json(), self.missing_authorization)

    def test_create_tokens_with_unknown_project_scope(self):
        """
        Verify token creation fails with unknown project and scope
        :return:
        """
        query_string = [('projectName', 'project_name_example'),
                        ('scope', 'scope_example')]

        # Create tokens with unknown project and scope
        response = requests.post(url=self.create_url, headers=self.headers, params=query_string, verify=False)
        self.assertEqual(401, response.status_code)
        self.assertEqual(response.json(), self.missing_authorization)

    def test_create_tokens_and_get_with_project_scope(self):
        """
        Verify token creation succeeds with known project and scope
        Verify get token without authentication fails
        :return:
        """
        query_string = [('projectName', 'RENCI-TEST'),
                        ('scope', 'measurement')]

        # Create tokens with project and scope
        response = requests.post(url=self.create_url, headers=self.headers, params=query_string, verify=False)
        self.assertEqual(401, response.status_code)
        self.assertEqual(response.json(), self.missing_authorization)

    def test_refresh_tokens(self):
        """
        Verify Refresh Tokens
        """
        # Invalid request
        response = requests.post(url=self.refresh_url, headers=self.headers, verify=False)
        self.assertEqual(400, response.status_code)

        # valid request but unknown token
        value = {"refresh_token":
                     "https://cilogon.org/oauth2/refreshToken/46438248f4b7691a851f88b0849d9687/1584383387474"}

        response = requests.post(url=self.refresh_url, headers=self.headers, json=value, verify=False)
        self.assertEqual(500, response.status_code)
        self.assertIsNotNone(response.json())

        query_string = [('projectName', 'RENCI-TEST'),
                        ('scope', 'measurement')]

        # valid request with project and scope but unknown token
        response = requests.post(url=self.refresh_url, headers=self.headers,
                                 json=value, params=query_string, verify=False)
        self.assertEqual(500, response.status_code)
        self.assertIsNotNone(response.json())

        query_string = [('projectName', 'project_name_example'),
                        ('scope', 'scope_example')]

        # valid request with unknonw project and scope but unknown token
        response = requests.post(url=self.refresh_url, headers=self.headers,
                                 json=value, params=query_string, verify=False)
        self.assertEqual(500, response.status_code)
        self.assertIsNotNone(response.json())

    def test_revoke_tokens(self):
        """
        Verify Revoke Tokens API
        """
        #invalid request
        response = requests.post(url=self.revoke_url, headers=self.headers, verify=False)

        self.assertEqual(400, response.status_code)

        value = {
            "refresh_token": "https://cilogon.org/oauth2/refreshToken/46438248f4b7691a851f88b0849d9687/1584383387474"}

        response = requests.post(url=self.revoke_url, headers=self.headers, json=value, verify=False)
        self.assertEqual(500, response.status_code)
        self.assertTrue(response.json().find("server_error") != -1)



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from fabric_cm.credmgr.swagger_server.models.base_model_ import Model
from fabric_cm.credmgr.swagger_server import util


class Request(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    def __init__(self, refresh_token: str=None):  # noqa: E501
        """Request - a model defined in Swagger

        :param refresh_token: The refresh_token of this Request.  # noqa: E501
        :type refresh_token: str
        """
        self.swagger_types = {
            'refresh_token': str
        }

        self.attribute_map = {
            'refresh_token': 'refresh_token'
        }
        self._refresh_token = refresh_token

    @classmethod
    def from_dict(cls, dikt) -> 'Request':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The request of this Request.  # noqa: E501
        :rtype: Request
        """
        return util.deserialize_model(dikt, cls)

    @property
    def refresh_token(self) -> str:
        """Gets the refresh_token of this Request.

        Refresh Token  # noqa: E501

        :return: The refresh_token of this Request.
        :rtype: str
        """
        return self._refresh_token

    @refresh_token.setter
    def refresh_token(self, refresh_token: str):
        """Sets the refresh_token of this Request.

        Refresh Token  # noqa: E501

        :param refresh_token: The refresh_token of this Request.
        :type refresh_token: str
        """
        if refresh_token is None:
            raise ValueError("Invalid value for `refresh_token`, must not be `None`")  # noqa: E501

        self._refresh_token = refresh_token



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
"""
Module responsible for handling Credmgr REST API logic
"""

import base64
import requests
from requests_oauthlib import OAuth2Session

from .abstract_credential_manager import AbstractCredentialManager
from fabric_cm.credmgr.config import CONFIG_OBJ
from fabric_cm.credmgr.logging import LOG
from fabric_cm.credmgr.token.fabric_token_encoder import FabricTokenEncoder
from fabric_cm.credmgr.swagger_server import jwt_validator
from fss_utils.jwt_manager import ValidateCode


class OAuthCredmgr(AbstractCredentialManager):
    """
    Credential Manager class responsible for handling various operations supported by REST APIs
    It also provides support for scanning and cleaning up of expired tokens and key files.
    """
    ID_TOKEN = "id_token"
    REFRESH_TOKEN = "refresh_token"
    ERROR = "error"
    CLIENT_ID = "client_id"
    CLIENT_SECRET = "client_secret"
    REVOKE_URI = "revoke_uri"
    TOKEN_URI = "token_uri"
    UTF_8 = "utf-8"

    def __init__(self):
        self.log = LOG

    def _generate_fabric_token(self, ci_logon_id_token: str, project: str,
                               scope: str, cookie: str = None):
        self.log.debug("CILogon Token: %s", ci_logon_id_token)
        # validate the token
        if jwt_validator is not None:
            LOG.info("Validating CI Logon token")
            code, claims_or_exception = jwt_validator.validate_jwt(token=ci_logon_id_token)
            if code is not ValidateCode.VALID:
                LOG.error(f"Unable to validate provided token: {code}/{claims_or_exception}")
                raise claims_or_exception

            fabric_token_encoder = FabricTokenEncoder(id_token=ci_logon_id_token, idp_claims=claims_or_exception,
                                                      project=project, scope=scope, cookie=cookie)

            validity = CONFIG_OBJ.get_token_life_time()
            private_key = CONFIG_OBJ.get_jwt_private_key()
            pass_phrase = CONFIG_OBJ.get_jwt_private_key_pass_phrase()
            kid = CONFIG_OBJ.get_jwt_public_key_kid()

            return fabric_token_encoder.encode(private_key=private_key, validity_in_seconds=validity, kid=kid,
                                               pass_phrase=pass_phrase)
        else:
            LOG.warning("JWT Token validator not initialized, skipping validation")

        return None

    def create_token(self, project: str, scope: str, ci_logon_id_token: str,
                     refresh_token: str, cookie: str = None) -> dict:
        """
        Generates key file and return authorization url for user to
        authenticate itself and also returns user id

        @param project: Project for which token is requested, by default it is set to 'all'
        @param scope: Scope of the requested token, by default it is set to 'all'
        @param ci_logon_id_token: CI logon Identity Token
        @param refresh_token: Refresh Token
        @param cookie: Vouch Proxy Cookie

        @returns dict containing id_token and refresh_token
        @raises Exception in case of error
        """

        self.validate_scope(scope=scope)

        if project is None or scope is None:
            raise OAuthCredMgrError("CredMgr: Cannot request to create a token, "
                                    "Missing required parameter 'project' or 'scope'!")

        id_token = self._generate_fabric_token(ci_logon_id_token=ci_logon_id_token,
                                               project=project, scope=scope,
                                               cookie=cookie)

        result = {self.ID_TOKEN: id_token, self.REFRESH_TOKEN: refresh_token}
        return result

    def refresh_token(self, refresh_token: str, project: str, scope: str, cookie: str = None) -> dict:
        """
        Refreshes a token from CILogon and generates Fabric token using project and scope saved in Database

        @param project: Project for which token is requested, by default it is set to 'all'
        @param scope: Scope of the requested token, by default it is set to 'all'
        @param refresh_token: Refresh Token
        @param cookie: Vouch Proxy Cookie
        @returns dict containing id_token and refresh_token

        @raises Exception in case of error
        """

        self.validate_scope(scope=scope)

        if OAuth2Session is None or refresh_token is None:
            raise ImportError("No module named OAuth2Session or refresh_token not provided")

        provider = CONFIG_OBJ.get_oauth_provider()
        providers = CONFIG_OBJ.get_providers()

        refresh_token_dict = {self.REFRESH_TOKEN: refresh_token}
        self.log.debug(f"Incoming refresh_token: {refresh_token}")

        # refresh the token (provides both new refresh and access tokens)
        oauth_client = OAuth2Session(providers[provider][self.CLIENT_ID], token=refresh_token_dict)
        new_token = oauth_client.refresh_token(providers[provider][self.TOKEN_URI],
                                               client_id=providers[provider][self.CLIENT_ID],
                                               client_secret=providers[provider][self.CLIENT_SECRET])

        new_refresh_token = None
        try:
            new_refresh_token = new_token.pop(self.REFRESH_TOKEN)
            id_token = new_token.pop(self.ID_TOKEN)
        except KeyError:
            self.log.error("No refresh or id token returned")
            raise OAuthCredMgrError("No refresh or id token returned")
        self.log.debug(f"new_refresh_token: {new_refresh_token}")

        try:
            id_token = self._generate_fabric_token(ci_logon_id_token=id_token,
                                                   project=project, scope=scope, cookie=cookie)
            result = {self.ID_TOKEN: id_token, self.REFRESH_TOKEN: new_refresh_token}

            return result
        except Exception as e:
            self.log.error(f"Exception error while generating Fabric Token: {e}")
            self.log.error(f"Failed generating the token but still returning refresh token")
            error_string = f"error: {str(e)}, {self.REFRESH_TOKEN}: {new_refresh_token}"
            raise OAuthCredMgrError(error_string)

    def revoke_token(self, refresh_token: str):
        """
        Revoke a refresh token

        @returns dictionary containing status of the operation
        @raises Exception in case of error
        """
        if OAuth2Session is None or refresh_token is None:
            raise ImportError("No module named OAuth2Session or revoke_token not provided")

        provider = CONFIG_OBJ.get_oauth_provider()
        providers = CONFIG_OBJ.get_providers()

        auth = providers[provider][self.CLIENT_ID] + ":" + providers[provider][self.CLIENT_SECRET]
        encoded_auth = base64.b64encode(bytes(auth, self.UTF_8))

        headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Authorization': 'Basic ' + str(encoded_auth, self.UTF_8)
        }

        data = f"token={refresh_token}&token_type_hint=refresh_token"

        response = requests.post(providers[provider][self.REVOKE_URI], headers=headers, data=data)
        self.log.debug("Response Status=%d", response.status_code)
        self.log.debug("Response Reason=%s", response.reason)
        self.log.debug(str(response.content, self.UTF_8))
        if response.status_code != 200:
            raise OAuthCredMgrError(str(response.content, self.UTF_8))

    @staticmethod
    def validate_scope(scope: str):
        allowed_scopes = CONFIG_OBJ.get_allowed_scopes()
        if scope not in allowed_scopes:
            raise OAuthCredMgrError(f"Scope {scope} is not allowed! Allowed scope values: {allowed_scopes}")


class OAuthCredMgrError(Exception):
    """
    Credmgr Exception
    """



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
"""
Main Entry Point
"""
import os
import signal

import connexion
import prometheus_client
import waitress
from flask import jsonify

from fabric_cm.credmgr.swagger_server import encoder
from fabric_cm.credmgr.config import CONFIG_OBJ
from fabric_cm.credmgr.logging import LOG


def main():
    """
    Main Entry Point
    """
    log = LOG
    try:
        app = connexion.App(__name__, specification_dir='swagger/')
        app.app.json_encoder = encoder.JSONEncoder
        app.add_api('swagger.yaml',
                    arguments={'title': 'Fabric Credential Manager API'},
                    pythonic_params=True)
        port = CONFIG_OBJ.get_rest_port()

        # prometheus server
        prometheus_port = CONFIG_OBJ.get_prometheus_port()
        prometheus_client.start_http_server(prometheus_port)

        # Start up the server to expose the metrics.
        waitress.serve(app, port=port)

    except Exception as ex:
        log.error("Exception occurred while starting Flask app")
        log.error(ex)
        raise ex


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
"""
Credential Manager
"""
import os
import sys
import configparser

CONF_DIR = '/etc/credmgr'
CONF_FILE = 'config'

PORT = '7000'
REST_PORT = '8100'

LOG_DIR = '/var/log/credmgr'
LOG_FILE = 'credmgr.log'
LOG_LEVEL = 'DEBUG'
LOG_RETAIN = '5'
LOG_FILE_SIZE = '5000000'

DEFAULT_TOKEN_LIFE_TIME = 60

LOGGER = 'logger'

CONFIG = configparser.ConfigParser()
CONFIG.add_section('runtime')
CONFIG.add_section('oauth')
CONFIG.add_section('logging')

CONFIG.set('runtime', 'port', PORT)
CONFIG.set('runtime', 'rest-port', REST_PORT)

CONFIG.set('logging', 'logger', LOGGER)
CONFIG.set('logging', 'log-directory', LOG_DIR)
CONFIG.set('logging', 'log-file', LOG_FILE)
CONFIG.set('logging', 'log-level', LOG_LEVEL)
CONFIG.set('logging', 'log-retain', LOG_RETAIN)
CONFIG.set('logging', 'log-file-size', LOG_FILE_SIZE)

# Now, attempt to read in the configuration file.
if os.getenv('TEST_ENVIRONMENT', 'False') == 'True':
    CONF_DIR = '../../../../docker'

CONFIG_FILE = CONF_DIR + '/' + CONF_FILE
try:
    files_read = CONFIG.read(CONFIG_FILE)
    if len(files_read) == 0:
        sys.stderr.write('Configuration file could not be read; proceeding with default settings.')
except Exception:
    raise RuntimeError('Unable to parse configuration file')



from datetime import timedelta

from fss_utils.jwt_manager import JWTManager, ValidateCode
from fss_utils.jwt_validate import JWTValidator
import prometheus_client

from fabric_cm.credmgr.config import CONFIG_OBJ
from fabric_cm.credmgr.logging import LOG

received_counter = prometheus_client.Counter('Requests_Received', 'HTTP Requests', ['method', 'endpoint'])
success_counter = prometheus_client.Counter('Requests_Success', 'HTTP Success', ['method', 'endpoint'])
failure_counter = prometheus_client.Counter('Requests_Failed', 'HTTP Failures', ['method', 'endpoint'])

# initialize CI Logon Token Validation
CILOGON_CERTS = CONFIG_OBJ.get_oauth_jwks_url()
CILOGON_KEY_REFRESH = CONFIG_OBJ.get_oauth_key_refresh()
LOG.info(f'Initializing JWT Validator to use {CILOGON_CERTS} endpoint, '
         f'refreshing keys every {CILOGON_KEY_REFRESH} HH:MM:SS')
jwt_validator = JWTValidator(url=CILOGON_CERTS,
                             refresh_period=timedelta(hours=CILOGON_KEY_REFRESH.hour,
                                                      minutes=CILOGON_KEY_REFRESH.minute,
                                                      seconds=CILOGON_KEY_REFRESH.second),
                             audience=CONFIG_OBJ.get_oauth_client_id())

kid = CONFIG_OBJ.get_jwt_public_key_kid()
public_key = CONFIG_OBJ.get_jwt_public_key()
code, jwk_public_key_rsa = JWTManager.encode_jwk(key_file_name=public_key, kid=kid, alg="RS256")
if code != ValidateCode.VALID:
    LOG.error("Failed to encode JWK")
    raise jwk_public_key_rsa

fabric_jwks = {"keys": [jwk_public_key_rsa]}



import connexion
import six

from fabric_cm.credmgr.swagger_server.models.request import Request  # noqa: E501
from fabric_cm.credmgr.swagger_server.models.success import Success  # noqa: E501
from fabric_cm.credmgr.swagger_server import util
from fabric_cm.credmgr.swagger_server.response import tokens_controller as rc


def tokens_create_post(project_name=None, scope=None):  # noqa: E501
    """Generate tokens for an user

    Request to generate tokens for an user  # noqa: E501

    :param project_name: Project Name
    :type project_name: str
    :param scope: Scope for which token is requested
    :type scope: str

    :rtype: Success
    """
    return rc.tokens_create_post(project_name=project_name, scope=scope)


def tokens_refresh_post(body=None, project_name=None, scope=None):  # noqa: E501
    """Refresh tokens for an user

    Request to refresh OAuth tokens for an user  # noqa: E501

    :param body: 
    :type body: dict | bytes
    :param project_name: Project Name
    :type project_name: str
    :param scope: Scope for which token is requested
    :type scope: str

    :rtype: Success
    """
    if connexion.request.is_json:
        body = Request.from_dict(connexion.request.get_json())  # noqa: E501
    return rc.tokens_refresh_post(body=body, project_name=project_name, scope=scope)


def tokens_revoke_post(body=None):  # noqa: E501
    """Revoke a refresh token for an user

    Request to revoke a refresh token for an user  # noqa: E501

    :param body: 
    :type body: dict | bytes

    :rtype: None
    """
    if connexion.request.is_json:
        body = Request.from_dict(connexion.request.get_json())  # noqa: E501
    return rc.tokens_revoke_post(body=body)



import connexion
import six

from fabric_cm.credmgr.swagger_server.models.jwks import Jwks  # noqa: E501
from fabric_cm.credmgr.swagger_server.models.version import Version  # noqa: E501
from fabric_cm.credmgr.swagger_server import util
from fabric_cm.credmgr.swagger_server.response import default_controller as rc


def certs_get():  # noqa: E501
    """Return Public Keys to verify signature of the tokens

    Json Web Keys # noqa: E501


    :rtype: Jwks
    """
    return rc.certs_get()


def version_get():  # noqa: E501
    """version

    Version # noqa: E501


    :rtype: Version
    """
    return rc.version_get()



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from fabric_cm.credmgr.swagger_server.models.base_model_ import Model
from fabric_cm.credmgr.swagger_server import util


class Version(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    def __init__(self, version: str=None, gitsha1: str=None):  # noqa: E501
        """Version - a model defined in Swagger

        :param version: The version of this Version.  # noqa: E501
        :type version: str
        :param gitsha1: The gitsha1 of this Version.  # noqa: E501
        :type gitsha1: str
        """
        self.swagger_types = {
            'version': str,
            'gitsha1': str
        }

        self.attribute_map = {
            'version': 'version',
            'gitsha1': 'gitsha1'
        }
        self._version = version
        self._gitsha1 = gitsha1

    @classmethod
    def from_dict(cls, dikt) -> 'Version':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The version of this Version.  # noqa: E501
        :rtype: Version
        """
        return util.deserialize_model(dikt, cls)

    @property
    def version(self) -> str:
        """Gets the version of this Version.


        :return: The version of this Version.
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version: str):
        """Sets the version of this Version.


        :param version: The version of this Version.
        :type version: str
        """

        self._version = version

    @property
    def gitsha1(self) -> str:
        """Gets the gitsha1 of this Version.


        :return: The gitsha1 of this Version.
        :rtype: str
        """
        return self._gitsha1

    @gitsha1.setter
    def gitsha1(self, gitsha1: str):
        """Sets the gitsha1 of this Version.


        :param gitsha1: The gitsha1 of this Version.
        :type gitsha1: str
        """

        self._gitsha1 = gitsha1



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from fabric_cm.credmgr.swagger_server.models.base_model_ import Model
from fabric_cm.credmgr.swagger_server.models.jwks_keys import JwksKeys  # noqa: F401,E501
from fabric_cm.credmgr.swagger_server import util


class Jwks(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    def __init__(self, keys: List[JwksKeys]=None):  # noqa: E501
        """Jwks - a model defined in Swagger

        :param keys: The keys of this Jwks.  # noqa: E501
        :type keys: List[JwksKeys]
        """
        self.swagger_types = {
            'keys': List[JwksKeys]
        }

        self.attribute_map = {
            'keys': 'keys'
        }
        self._keys = keys

    @classmethod
    def from_dict(cls, dikt) -> 'Jwks':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The jwks of this Jwks.  # noqa: E501
        :rtype: Jwks
        """
        return util.deserialize_model(dikt, cls)

    @property
    def keys(self) -> List[JwksKeys]:
        """Gets the keys of this Jwks.


        :return: The keys of this Jwks.
        :rtype: List[JwksKeys]
        """
        return self._keys

    @keys.setter
    def keys(self, keys: List[JwksKeys]):
        """Sets the keys of this Jwks.


        :param keys: The keys of this Jwks.
        :type keys: List[JwksKeys]
        """

        self._keys = keys



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
"""
Provides logging functions
"""
import logging
import os
from logging.handlers import RotatingFileHandler

from fabric_cm.credmgr.config import CONFIG_OBJ


def get_logger():
    """
    Detects the path and level for the log file from the credmgr CONFIG_OBJ and sets
    up a logger. Instead of detecting the path and/or level from the
    credmgr CONFIG_OBJ, a custom path and/or level for the log file can be passed as
    optional arguments.

    :param log_path: Path to custom log file
    :param log_level: Custom log level
    :return: logging.Logger object
    """

    # Get the log path
    log_path = CONFIG_OBJ.get_logger_dir() + '/' + CONFIG_OBJ.get_logger_file()
    if log_path is None:
        raise RuntimeError('The log file path must be specified in CONFIG_OBJ or passed as an argument')

    # Get the log level
    log_level = CONFIG_OBJ.get_logger_level()
    if log_level is None:
        log_level = logging.INFO

    logger = CONFIG_OBJ.get_logger_name()

    # Set up the root logger
    log = logging.getLogger(logger)
    log.setLevel(log_level)
    log_format = '%(asctime)s - %(name)s - {%(filename)s:%(lineno)d} - %(levelname)s - %(message)s'
    os.makedirs(os.path.dirname(log_path), exist_ok=True)

    file_handler = RotatingFileHandler(log_path,
                                       backupCount=CONFIG_OBJ.get_logger_retain(),
                                       maxBytes=CONFIG_OBJ.get_logger_size())

    logging.basicConfig(handlers=[file_handler], format=log_format)

    return log, file_handler



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
# coding: utf-8

from setuptools import setup, find_packages

from fabric_cm.credmgr import CONF_DIR, CONF_FILE, LOG_DIR

NAME = "fabric-credmgr"
VERSION = "0.9"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = [
        'requests',
        'requests_oauthlib',
        'connexion',
        'swagger-ui-bundle',
        'python_dateutil',
        'setuptools',
        'psycopg2-binary',
        'sqlalchemy',
        'PyJWT',
        'ldap3',
        'prometheus_client',
        'waitress',
        'six',
        'cryptography'
        ]

setup(
    name=NAME,
    version = VERSION,
    description="Fabric Credential Manager API",
    author_email="kthare10@renci.org",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url='https://github.com/fabric-testbed/CredentialManager',
    author='Komal Thareja',
    keywords=["Swagger", "Fabric Credential Manager API"],
    license='MIT',
    install_requires = REQUIRES,
    packages=find_packages(),
    package_data={'': ['swagger/swagger.yaml']},
    include_package_data=True,
    data_files = [(CONF_DIR, [CONF_FILE]), (LOG_DIR, [])],
    entry_points={
        'console_scripts': ['fabric_cm.credmgr.swagger_server=fabric_cm.credmgr.swagger_server.__main__:main']},
    classifiers=[
                  "Programming Language :: Python :: 3",
                  "License :: OSI Approved :: MIT License",
                  "Operating System :: OS Independent",
              ],
    python_requires='>=3.7'
)



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from fabric_cm.credmgr.swagger_server.models.base_model_ import Model
from fabric_cm.credmgr.swagger_server import util


class JwksKeys(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    def __init__(self, kty: str=None, e: str=None, n: str=None, use: str=None, alg: str=None, kid: str=None):  # noqa: E501
        """JwksKeys - a model defined in Swagger

        :param kty: The kty of this JwksKeys.  # noqa: E501
        :type kty: str
        :param e: The e of this JwksKeys.  # noqa: E501
        :type e: str
        :param n: The n of this JwksKeys.  # noqa: E501
        :type n: str
        :param use: The use of this JwksKeys.  # noqa: E501
        :type use: str
        :param alg: The alg of this JwksKeys.  # noqa: E501
        :type alg: str
        :param kid: The kid of this JwksKeys.  # noqa: E501
        :type kid: str
        """
        self.swagger_types = {
            'kty': str,
            'e': str,
            'n': str,
            'use': str,
            'alg': str,
            'kid': str
        }

        self.attribute_map = {
            'kty': 'kty',
            'e': 'e',
            'n': 'n',
            'use': 'use',
            'alg': 'alg',
            'kid': 'kid'
        }
        self._kty = kty
        self._e = e
        self._n = n
        self._use = use
        self._alg = alg
        self._kid = kid

    @classmethod
    def from_dict(cls, dikt) -> 'JwksKeys':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The jwks_keys of this JwksKeys.  # noqa: E501
        :rtype: JwksKeys
        """
        return util.deserialize_model(dikt, cls)

    @property
    def kty(self) -> str:
        """Gets the kty of this JwksKeys.

        Key Type  # noqa: E501

        :return: The kty of this JwksKeys.
        :rtype: str
        """
        return self._kty

    @kty.setter
    def kty(self, kty: str):
        """Sets the kty of this JwksKeys.

        Key Type  # noqa: E501

        :param kty: The kty of this JwksKeys.
        :type kty: str
        """

        self._kty = kty

    @property
    def e(self) -> str:
        """Gets the e of this JwksKeys.

        Exponent Parameter  # noqa: E501

        :return: The e of this JwksKeys.
        :rtype: str
        """
        return self._e

    @e.setter
    def e(self, e: str):
        """Sets the e of this JwksKeys.

        Exponent Parameter  # noqa: E501

        :param e: The e of this JwksKeys.
        :type e: str
        """

        self._e = e

    @property
    def n(self) -> str:
        """Gets the n of this JwksKeys.

        Modulus Parameter  # noqa: E501

        :return: The n of this JwksKeys.
        :rtype: str
        """
        return self._n

    @n.setter
    def n(self, n: str):
        """Sets the n of this JwksKeys.

        Modulus Parameter  # noqa: E501

        :param n: The n of this JwksKeys.
        :type n: str
        """

        self._n = n

    @property
    def use(self) -> str:
        """Gets the use of this JwksKeys.

        Public Key Use Parameter  # noqa: E501

        :return: The use of this JwksKeys.
        :rtype: str
        """
        return self._use

    @use.setter
    def use(self, use: str):
        """Sets the use of this JwksKeys.

        Public Key Use Parameter  # noqa: E501

        :param use: The use of this JwksKeys.
        :type use: str
        """

        self._use = use

    @property
    def alg(self) -> str:
        """Gets the alg of this JwksKeys.

        Algorithm Parameter  # noqa: E501

        :return: The alg of this JwksKeys.
        :rtype: str
        """
        return self._alg

    @alg.setter
    def alg(self, alg: str):
        """Sets the alg of this JwksKeys.

        Algorithm Parameter  # noqa: E501

        :param alg: The alg of this JwksKeys.
        :type alg: str
        """

        self._alg = alg

    @property
    def kid(self) -> str:
        """Gets the kid of this JwksKeys.

        Key Id Header Parameter  # noqa: E501

        :return: The kid of this JwksKeys.
        :rtype: str
        """
        return self._kid

    @kid.setter
    def kid(self, kid: str):
        """Sets the kid of this JwksKeys.

        Key Id Header Parameter  # noqa: E501

        :param kid: The kid of this JwksKeys.
        :type kid: str
        """

        self._kid = kid



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author Komal Thareja (kthare10@renci.org)
"""
Base class for Credential Manager
"""
from abc import ABCMeta, abstractmethod
import six


@six.add_metaclass(ABCMeta)
class AbstractCredentialManager:
    """
    Abstract Credential Manager class
    """
    @abstractmethod
    def create_token(self, project: str, scope: str, ci_logon_id_token: str,
                     refresh_token: str, cookie: str = None) -> dict:
        """
        Generates key file and return authorization url for user to authenticate itself and also returns user id

        @param project: Project for which token is requested, by default it is set to 'all'
        @param scope: Scope of the requested token, by default it is set to 'all'
        @param ci_logon_id_token: CI logon Identity Token
        @param refresh_token: Refresh Token
        @param cookie: Vouch Proxy Cookie

        @returns dict containing id_token and refresh_token
        @raises Exception in case of error
        """

    @abstractmethod
    def refresh_token(self, refresh_token: str, project: str, scope: str, cookie: str = None) -> dict:
        """
        Refreshes a token from CILogon and generates Fabric token using project and scope saved in Database

        @param project: Project for which token is requested, by default it is set to 'all'
        @param scope: Scope of the requested token, by default it is set to 'all'
        @param refresh_token: Refresh Token
        @param cookie: Vouch Proxy Cookie
        @returns dict containing id_token and refresh_token

        @raises Exception in case of error
        """

    @abstractmethod
    def revoke_token(self, refresh_token: str):
        """
        Revoke a refresh token

        @returns dictionary containing status of the operation
        @raises Exception in case of error
        """
